/* Args.C <-- A module dealing with arguments */

#include <string.h>
#include <stdlib.h>
#include <ncurses.h>
#include <term.h>
#include "classes.h"
#include "defines.h"

bool to_break;

bool is_string_int (const char *in)
 { 
 for (unsigned cx = 0; cx < strlen (in); cx++)
   if ( in[cx] > '9' || in[cx] < '0') return 0; 
 return 1;
 }

void show_stdin_err ( int what )
 {
 switch (what)
  {
  case 1:
   printf("stdin structure error:\nMore info with \"saper --help\"\n");
   break;
  case 2:
   printf("stdin structure error: invalid board geometry\n");
   break;
  case 3: 
   printf("terminal error: invalid term geometry\n");
   printf("saper needs to at least 73x10\n");
   break;
  }

 if ( what != 0 )
  to_break = 1;
 }

void show_help (void)
 {
 printf("SAPER - saper-%s (%s, compiled on %s %s)\n\n",VERSION,DATE,__DATE__,__TIME__);
 printf("Usage: saper [flags] [arguments]	\n");
 printf("Arguments:\n");
 printf("   --help		summarize saper usage. (same as -h)\n");
 printf("   --version		print version information. (same as -v )\n");
 printf("   --bombs=n		set the bomb number to n. (same as -bn )\n");
 printf("   --geometry=WxH	set geometry of the main board. (same as -gWxH)\n");
 printf("   --lang=LANGUAGE	set language of interface. (same as -lLANGUAGE)\n");
 printf("Flags:\n");
 printf("   [+/-]ai		[enable/disable] AI (enable is default).\n");
 printf("   euclid		set the board on Euclid's surface. (default)\n");
 printf("   torus		set the board on Torus surface. \n");
 printf("   klein		set the board on Klein's bottle. \n"); 
 printf("Avaible languages:\n");
 printf("   English	(eng)	by olorin@staszic.waw.pl\n");
 printf("   Polish	(pol)	by olorin@staszic.waw.pl\n");
 printf("\nPlease send any bug reports to FHF (see --FHF for more info).\n");
 }
 
void show_version (void)
 {
 printf("%s\n",VERSION);
 }

void show_FHF (void) {
	printf("Copyright (C) [2002; 2004] Free Hackers Foundation (FHF).\n\n");
	printf("	Jedrzej Jablonski (MateX2003)		[main code]\n");
	printf("	    olorin@staszic.waw.pl\n\n");
	printf("	Marek Marczykowski (MateX2003)		[AI]\n");
	printf("	   marmarek@staszic.waw.pl\n\n");
	printf("	Krzysztof Sachanowicz (MateX2003) 	[error corr]\n");
	printf("	    krzysio2@staszic.waw.pl\n\n");
}

int set_lang ( const char *argv, unsigned len )
{
 char *name;
 name = (char*) argv + len;
 if ( strlen (argv) == len ) {
  show_stdin_err (1);
  return 0;
 }
 else if ( strcmp(name,"eng")==0 || strcmp(name,"English")==0 || strcmp(name,"english")==0)
	 return 1;
 else if ( strcmp(name,"pol")==0 || strcmp(name,"Polish")==0 || strcmp(name,"polish")==0)
	 return 2; 
 else {
	 show_stdin_err(1);
	 return 0;
 }
}
int set_bombs ( const char *argv, unsigned len )
{
 char number[4];
 if ( strlen (argv) == len || strlen (argv) >= len+4 ) {
  show_stdin_err (1);
  return -1;
 }
 else 
  {
  for (unsigned cx = len; cx <= strlen (argv); cx++) number[cx - len] = argv[cx];
    return atoi(number);
  }
}
 
XY set_geometry (const char *argv, int len)
{
 XY tmp;
 if (sscanf(argv+len,"%dx%d",&tmp.x,&tmp.y)!=2 || tmp.x<=0 || tmp.y<=0) {
    show_stdin_err (1);
    tmp = XY(-1,-1);
 }
 else if (tmp.x < 4 || tmp.y < 4)
  {
  show_stdin_err(2);
  tmp = XY(-1,-1);
  }
 return tmp;
}


bool handle_args (int argc, const char **argv, defs& defaults)
 {
 setupterm(NULL, fileno(stdout), (int *) 0);
 to_break = 0;
 if(tigetnum("cols") < 73 || tigetnum("lines") < 10 ) 
  {
  to_break = 1; 
  show_stdin_err(3);
  } 
 for (int cx = 1; cx < argc && !to_break; cx++)
  {
  if ( strcmp (argv[cx], "--help" ) == 0)		{ to_break = 1; show_help (); }
  else if ( strcmp (argv[cx], "-h" ) == 0)		{ to_break = 1; show_help (); }
  else if ( strcmp (argv[cx], "--version" ) == 0)	{ to_break = 1; show_version (); }
  else if ( strcmp (argv[cx], "-v" ) == 0)		{ to_break = 1; show_version (); }
  else if ( strcmp (argv[cx], "--FHF" ) == 0)		{ to_break = 1; show_FHF (); }
  else if ( strcmp (argv[cx], "-ai" ) == 0 )		defaults.ai = 0;
  else if ( strcmp (argv[cx], "+ai" ) == 0 )		defaults.ai = 1;
  else if ( strcmp (argv[cx], "euclid" ) == 0 )		defaults.surface = 1;
  else if ( strcmp (argv[cx], "klein" ) == 0 )		defaults.surface = 2;
  else if ( strcmp (argv[cx], "torus" ) == 0 )		defaults.surface = 3;  
  else if ( strncmp(argv[cx], "--geometry=", 11) == 0 )	defaults.geometry = set_geometry (argv[cx],11);
  else if ( strncmp(argv[cx], "-g", 2) == 0 )		defaults.geometry = set_geometry (argv[cx],2);
  else if ( strncmp(argv[cx], "--bombs=", 8) == 0 )	defaults.total = set_bombs ( argv[cx], 8 );
  else if ( strncmp(argv[cx], "-b", 2) == 0 )	        defaults.total = set_bombs ( argv[cx], 2);
  else if ( strncmp(argv[cx], "--lang=", 7) == 0 )	defaults.lang = set_lang ( argv[cx], 7);
  else if ( strncmp(argv[cx], "-l", 2) == 0 )	        defaults.lang = set_lang ( argv[cx], 2);
  else show_stdin_err (1);
  }
 return !to_break;
 }

/* Copyright by FHF 2004 */
