/* Boxes.C <-- All functions operating on boxes */

#include <ncurses.h>
#include "classes.h" 
#include "../src/operations.h"
#include "defines.h"

int box_normal (WINDOW *win, int where)
 {
 switch (where)
  {
  case 1:
    return wborder (win, UD, UD, RL, RL, TR, PL, DL, TU);
  case 2:
    return wborder (win, UD, UD, RL, RL, PL, PL, TU, TU);
  case 3:
    return wborder (win, UD, UD, RL, RL, PL, TL, TU, DR);
  case 4:
    return wborder (win, UD, UD, RL, RL, TR, PL, TR, PL);  
  case 5:
    return wborder (win, UD, UD, RL, RL, PL, PL, PL, PL);
  case 6:
    return wborder (win, UD, UD, RL, RL, PL, TL, PL, TL);
  case 7:
    return wborder (win, UD, UD, RL, RL, UL, TD, TR, PL);
  case 8:
    return wborder (win, UD, UD, RL, RL, TD, TD, PL, PL);    
  case 9:
    return wborder (win, UD, UD, RL, RL, TD, UR, PL, TL);
  }
 return -1;
 }
 
int box_high (WINDOW *win)
 {
 return wborder (win, HSB, HSB, HSB, HSB, HSB, HSB, HSB, HSB); 
 }
 
void char_put (WINDOW *box_window, char x, int color)
 {
 int bold = 0;
 if ( color % 2 == 0 ) bold = A_BOLD;
 mvwaddch (box_window, 1, 1, ' ' | COLOR_PAIR (color) ); 
 mvwaddch (box_window, 1, 2,  x  | COLOR_PAIR (color) | bold);
 mvwaddch (box_window, 1, 3, ' ' | COLOR_PAIR (color) );
 }
 
void box_put_general (XY putxy, the_board& game, bool hidden)
 {
 if ( ! hidden ) {
  if (game.cursor == putxy) box_high (game.win(putxy));  
  else box_normal (game.win(putxy), putxy.whereis(game)); 
  }
 
 if (game.free(putxy) == 1)  char_put (game.win(putxy), 'P', 10);
 else if (game.seen(putxy) == 1)
   {
   if (game.num(putxy) > 0 && game.num(putxy) < 9) char_put (game.win(putxy), game.num(putxy) + '0', game.num(putxy) );
   else if (game.num(putxy) == 0) char_put (game.win(putxy), ' ', 1);
   else if (game.num(putxy) == 9) char_put (game.win(putxy), '*', 9);
   }
 if (game.free(putxy) && game.seen(putxy))
   {
   if (game.num(putxy) == 9)
     char_put (game.win(putxy),'P',10);
   else
     char_put (game.win(putxy),'X',10);
   }
 else if (!(game.free(putxy) | game.seen(putxy))) 
   char_put (game.win(putxy),' ',0);
 if (!hidden) 
   wrefresh (game.win(putxy));   
 }
 
void box_put (XY putxy, the_board& game)
 {
 box_put_general (putxy,game,0);
 }

void box_put_hidden (XY putxy, the_board& game)
 {
 box_put_general (putxy,game,1);
 } 
 
void box_move (int where, the_board& game)
 {
 XY oldcursor = game.cursor;
 game.cursor = game.cursor.move (game, where, surf2curs(game.defaults.surface));

 box_put (oldcursor, game);
 box_put (game.cursor, game); 
 }
 
void box_move_hidden (int where, the_board& game)
 {
 XY oldcursor = game.cursor;
 game.cursor = game.cursor.move (game, where, surf2curs(game.defaults.surface));

 box_put_hidden (oldcursor, game);
 box_put_hidden (game.cursor, game); 
 }

/* Copyright by FHF 2004 */
