/* ClassTheboard.C <-- class the_board definitions. */

#include <stdlib.h>
#include <time.h>
#include "defines.h"
#include "boxes.h"
#include "../src/init.h"
#include "../src/input.h"
#include "../src/operations.h"
#include "../src/lang.h"

void the_board::show_all ()
 {
 if ( !generated )
   generate ();
 for (int cy = 0; cy < Ysize; cy++)
  for (int cx = 0; cx < Xsize; cx++)
   {
   force(XY(cx,cy),1);
   box_put (XY(cx,cy),*this);
   }
 box_put (cursor,*this);
 }

void the_board::refresh_all () 
 {
 for (int cy = 0; cy < Ysize; cy++) 
   for (int cx = 0; cx < Xsize; cx++) 
      {
      touchwin(win(XY(cx,cy)));
      wrefresh(win(XY(cx,cy)));
      }
 box_put (cursor,*this);
 footer.make_all(*this);
 }

void the_board::startwin (void)
 {
 for (int cx = 0; cx < Xsize; cx++ )	// MAKING WINDOWS
   for (int cy = 0; cy < Ysize; cy++ )
     {
     windows[cy][cx] = newwin(3,5,2*cy,4*cx);
     box_put(XY(cx,cy), *this);
     }
  footer.statuswin = newwin(0,4*Xsize+1,2*Ysize+1,0);
  box_put(cursor, *this);
 }
 
void the_board::make (int Y, int X)
 {
 Xsize = X;
 Ysize = Y;

 int cx, cy;

 board = (box_type **) malloc ( Ysize * sizeof(box_type *) );
 for (cx = 0; cx < Ysize; cx++ )
   board[cx] = (box_type *) malloc (Xsize * sizeof(box_type) );
   
 windows = (WINDOW ***) malloc ( Ysize * sizeof(WINDOW **) );
 for (cx = 0; cx < Ysize; cx++ )
    windows[cx] = (WINDOW **) malloc (Xsize * sizeof(WINDOW *) );    
   
 for (cx = 0; cx < Xsize; cx++) for (cy = 0; cy < Ysize; cy++)
  {
  board[cy][cx].num = 0;
  board[cy][cx].seen = 0;
  board[cy][cx].free = 0;
  board[cy][cx].autorec = 0;  
  board[cy][cx].autorec2 = 0;
  }
  
 total = 0;
 orient = 0;
 cracked = 0;
 uncovered = 0;
 generated = 0;
 }
 
void the_board::generate ()
  {
  XY work;
  
  for (int x = 0; x < total; x ++)
    {
    do  { work = XY ( rand() % Xsize, rand() % Ysize ); }
    while ( num(work) == 9 || (work == cursor && total != Xsize*Ysize) );
    
    board[work.y][work.x].num = 9;
    
    for (int cx = 1; cx <= 9; cx ++)
      if (work.move(*this,cx,surf2move(defaults.surface)).x >= 0)
       bomb_inc ( board[work.move(*this,cx,surf2move(defaults.surface)).y][work.move(*this,cx,surf2move(defaults.surface)).x].num );
    }
  generated = 1;
  }
  
void defs::make (the_board &game)
 {
 int capx = (COLS - 1) /4;
 if(game.defaults.geometry.x!=-1 && game.defaults.geometry.x<capx)
    capx=game.defaults.geometry.x;
 int capy = (LINES - 2) /2;
 if(game.defaults.geometry.y!=-1 && game.defaults.geometry.y<capy)
    capy=game.defaults.geometry.y;

 game.make (capy, capx);
 if ( game.defaults.total >=0 ) 
   game.total = game.defaults.total;
 else game.total = capx*capy/4;
 if(game.total>capx*capy-2)
    game.total=capx*capy-2;
 if(game.total<0)
    game.total=0;
 if(game.total>999)
    game.total=999;

 game.defaults.surface += (game.defaults.surface == -1) ? 2 : 0;
 game.defaults.ai = (game.defaults.ai == -1) ? 1 : game.defaults.ai;

 game.startwin (); 
 quit=0;
 if (lang==0) lang=1;
 }

void the_board::newstart (void)
 {
  ::start ();			// init of srand, ncurses etc.
  start = time(NULL);
  defaults.make (*this);	// args ---> game
  footer.make_basic(*this); 	// make status bar
  footer.copyright(*this); 
  finished = 0;
 }
 
bool the_board::pause (void)
 {
 WINDOW *pausewin;
 time_t startp = time(NULL);
 char in;
 touchwin(stdscr);
 wrefresh(stdscr);
 touchwin(footer.statuswin);
 wrefresh(footer.statuswin); 
 pausewin = newwin(11,41,Ysize-10,Xsize*2-19);
 box(pausewin,UD,RL);
 mvwprintw(pausewin,5,10,_("pause",defaults.lang)); 
 wrefresh(pausewin);
 do 
 {
 in = wgetch(stdscr);
 }
 while ( in != 'p' && in != 'q');
 delwin(pausewin);
 start += (int) difftime(time(NULL),startp);
 if (in == 'p') 
   refresh_all();
 return (in == 'p') ? 0 : 1;
 }

void the_board::finish (void)
 {
 int inch;
 if (!finished)  {
   finished = 1;
   show_all();
   cracked=total;
   uncovered=Xsize*Ysize-cracked;
   footer.lost(*this);
   do {
	inch = char_get(win(cursor));
	footer.quest(*this);
   } while (!(inch==16||inch==64||inch==254||inch==255));
   if (inch==254||inch==64)
     defaults.quit=1;
   else {
	   newstart();
	   finished=1;
   }
   }
 }

void the_board::winner (void) {
	if(cracked!=total||Xsize*Ysize!=uncovered+cracked) return;
	int inch;
	footer.won(*this);
	do {
		inch = char_get(win(cursor));
		footer.quest(*this);
	} while (!(inch==16||inch==64||inch==254||inch==255));
	if (inch==254||inch==64)
		defaults.quit=1;
	else {
		newstart();
	}
}
  
/* Copyright by FHF 2004 */
